/* AllThingsTalk Arduino WiFi SDK
 * https://github.com/allthingstalk/arduino-wifi-sdk
 *
 * About this example:
 * Waits for you to send a message from your AllThingsTalk Maker from an Actuator asset
 * and, once received, relays that message (using CBOR) back to a specified Sensor asset
 * on your AllThingsTalk Maker.
 * This example will automatically create an actuator asset "relay-msg-actuator-example" on your AllThingsTalk for you to send a message from
 * This example will also automatically create a sensor asset "relay-msg-sensor-example" on your AllThingsTalk for you to receive a message on
 *
 * Notes:
 * - Create a device on your https://maker.allthingstalk.com (if you don't already have it)
 *
 * These are all the things in this example that you need to change to make it work:
 *   WiFiSSID, WiFiPassword, DeviceID, DeviceToken
 */

#include <AllThingsTalk_WiFi.h>
#include <OneWire.h>

OneWire  ds(5);      //  these are all the I2C grove connectors, so we can attach multiple onewire sensors :-)


int Solar_Relay = D8;                                        // Digital Actuator is connected to pin D0 on nodemcu -> built in led


auto wifiCreds   = WifiCredentials("WiFiSSID", "WiFiPassword");   // Your WiFi Network Name and Password
auto deviceCreds = DeviceConfig("DeviceID", "DeviceToken");       // Go to AllThingsTalk Maker > Devices > Your Device > Settings > Authentication to get your Device ID and Token
auto device      = Device(wifiCreds, deviceCreds);                // Create "device" object
char* actuator   = "8";                  // Name of asset on AllThingsTalk that you'll use to send a message (automatically created below)
CborPayload payload;                                              // Create CBOR payload object, so we can use CBOR to send data

void setup() {
  pinMode(Solar_Relay, OUTPUT);           // set pin to input
  Serial.begin(115200);                   // Baud rate: 115200, but you can define any baud rate you want
  device.debugPort(Serial);               // Set AllThingsTalk library to output its debug to "Serial"
  device.setActuationCallback(actuator, actuation);  // Actuator asset defined above will trigger function "actuation"
  //device.createAsset("relay-msg-actuator-example", "Relay Message Actuator (SDK Example)", "actuator", "string"); // Create asset on AllThingsTalk to send message
  //device.createAsset("relay-msg-sensor-example", "Relay Message Sensor (SDK Example)", "sensor", "string"); // Create asset on AllThingsTalk to receive message
  device.init();                          // Initialize WiFi and AllThingsTalk
}


// ONEWIRE FUNCTION
// READS TEMPERATURE FROM ONEWIRE TEMP SENSORS


void TempSample() {
  byte i;
  byte present = 0;
  byte type_s;
  byte data[12];
  byte addr[8];
  double celsius, fahrenheit;
  boolean EOS = false; //End of scan, All Onewiresensors are scanned.
  int SensorNr = 0;  //Number of onewire digital sensor


  while (EOS == false) {
     if ( !ds.search(addr)) {
       Serial.println("No more addresses.");
       EOS = true;
       Serial.println();
       ds.reset_search();
       SensorNr = 0;
        delay(250);
        return;
      }

  Serial.print("ROM =");
  for( i = 0; i < 8; i++) {
    Serial.write(' ');
    Serial.print(addr[i], HEX);
  }


  SensorNr = SensorNr+1;

  if (OneWire::crc8(addr, 7) != addr[7]) {
      Serial.println("CRC is not valid!");
      return;
  }
  Serial.println();

  // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
      Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
      Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
      Serial.println("Device is not a DS18x20 family device.");
      return;
  }

  ds.reset();
  ds.select(addr);
  ds.write(0x44, 1);        // start conversion, with parasite power on at the end

  delay(1000);     // maybe 750ms is enough, maybe not
  // we might do a ds.depower() here, but the reset will take care of it.

  present = ds.reset();
  ds.select(addr);
  ds.write(0xBE);         // Read Scratchpad

  Serial.print("  Data = ");
  Serial.print(present, HEX);
  Serial.print(" ");
  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
    Serial.print(data[i], HEX);
    Serial.print(" ");
  }
  Serial.print(" CRC=");
  Serial.print(OneWire::crc8(data, 8), HEX);
  Serial.println();

  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3; // 9 bit resolution default
    if (data[7] == 0x10) {
      // "count remain" gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);
    // at lower res, the low bits are undefined, so let's zero them
    if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
    //// default is 12 bit resolution, 750 ms conversion time
  }
  celsius = (float)raw / 16.0;
  fahrenheit = celsius * 1.8 + 32.0;
  Serial.print("  Temperature = ");
  Serial.print(celsius);
  Serial.print(" Celsius, ");
  Serial.print(fahrenheit);
  Serial.println(" Fahrenheit");

  // Send Temp to cloudplatform

  char TempString[10];  //  Hold The Convert Data
  dtostrf(celsius,3,2,TempString);

  if(addr[7] == 0xD3)
  device.send("3", String(TempString));
  if(addr[7] == 0x26)
  device.send("4", String(TempString));
  if(addr[7] == 0x80)
  device.send("5", String(TempString));
    if(addr[7] == 0x64)
  device.send("5", String(TempString));
  }
}


unsigned long timer;

void loop()
{
  unsigned long curTime = millis();
  if (curTime > (timer + 60000))    {                     // Sample each minute
  TempSample();
  Serial.print("time = ");Serial.println(curTime);
  timer = curTime;
  }
  device.loop();
}

void actuation(bool value) {            // Function called when message arrives on asset "actuator" defined above
  Serial.print("Received message: ");     // Prints to serial output
  Serial.println(value);                  // Prints actual value received to serial output
        if (value == 0) {
        digitalWrite(Solar_Relay, LOW);
        device.send("8", "false");
      }
      else if (value == 1) {
        digitalWrite(Solar_Relay, HIGH);
        device.send("8", "true");
      }


}
